
/*
	Password Prompter - PC Magazine password utility

	Copyright (c) 1999 Ziff-Davis Publishing Company.  All rights reserved.
	First published in PC Magazine, US Edition  June 8, 1999.

	Written by Steven E. Sipe


	This file implements the random password generation dialog class.
*/

#include "stdafx.h"
#include "passprom.h"
#include "PassDlg.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CPPGenPassDlg dialog

CPPGenPassDlg::CPPGenPassDlg(CWnd* pParent /*=NULL*/)
	: CDialog(CPPGenPassDlg::IDD, pParent)
{
	//{{AFX_DATA_INIT(CPPGenPassDlg)
	m_Letters = FALSE;
	m_Lower = FALSE;
	m_NewPassword = _T("");
	m_Numbers = FALSE;
	m_Symbols = FALSE;
	m_Upper = FALSE;
	m_Length = 0;
	m_StartsWith = -1;
	//}}AFX_DATA_INIT
}


void CPPGenPassDlg::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CPPGenPassDlg)
	DDX_Control(pDX, IDOK, m_OK);
	DDX_Control(pDX, IDC_LENGTH_SPIN, m_LengthSpin);
	DDX_Check(pDX, IDC_LETTERS, m_Letters);
	DDX_Check(pDX, IDC_LOWER, m_Lower);
	DDX_Text(pDX, IDC_NEW_PASSWORD, m_NewPassword);
	DDV_MaxChars(pDX, m_NewPassword, 50);
	DDX_Check(pDX, IDC_NUMBERS, m_Numbers);
	DDX_Check(pDX, IDC_SYMBOLS, m_Symbols);
	DDX_Check(pDX, IDC_UPPER, m_Upper);
	DDX_Text(pDX, IDC_LENGTH, m_Length);
	DDV_MinMaxInt(pDX, m_Length, 5, 50);
	DDX_Radio(pDX, IDC_START_LETTER, m_StartsWith);
	//}}AFX_DATA_MAP
}


BEGIN_MESSAGE_MAP(CPPGenPassDlg, CDialog)
	//{{AFX_MSG_MAP(CPPGenPassDlg)
	ON_BN_CLICKED(IDC_GENERATE, OnGenerate)
	ON_BN_CLICKED(IDC_HELPBTN, OnHelpBtn)
	ON_WM_DESTROY()
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CPPGenPassDlg message handlers

// Initializes default field values for the random password generation
// screen
BOOL CPPGenPassDlg::OnInitDialog() 
{
	m_Length = AfxGetApp()->GetProfileInt("Passgen","Length",8);
	if(m_Length < 5) m_Length = 5;

	m_StartsWith = AfxGetApp()->GetProfileInt("Passgen","StartsWith",0);
	m_Upper = AfxGetApp()->GetProfileInt("Passgen","Upper",1);
	m_Lower = AfxGetApp()->GetProfileInt("Passgen","Lower",1);;
	m_Letters = AfxGetApp()->GetProfileInt("Passgen","Letters",1);;
	m_Numbers = AfxGetApp()->GetProfileInt("Passgen","Numbers",1);;
	m_Symbols = AfxGetApp()->GetProfileInt("Passgen","Symbols",0);;

	((CEdit *)GetDlgItem(IDC_LENGTH))->LimitText(2);

	CDialog::OnInitDialog();

	m_LengthSpin.SetRange(5,50);

	return TRUE;  // return TRUE unless you set the focus to a control
	              // EXCEPTION: OCX Property Pages should return FALSE
}

// Handles the Generate button -- generates a new random password
void CPPGenPassDlg::OnGenerate() 
{
	char *szLetters = "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz";
	char *szNumbers = "0123456789";
	char *szSymbols = "_!@#$%^&*()+=?<>{}[]";
	char ch, ch2;
	int nIndex;
	BOOL bUpper = FALSE, bLower = FALSE;

	if(!UpdateData(TRUE))
		return;

	// Make sure Continue With has at least one selection
	if(!m_Letters && !m_Numbers && !m_Symbols)
	{
		AfxMessageBox(IDS_CHOOSE_CONTINUE,MB_ICONWARNING|MB_OK);
		return;
	}

	// If letters are chosen then make sure Case is selected
	if((m_StartsWith == 0 || m_Letters) && !m_Upper && !m_Lower)
	{
		AfxMessageBox(IDS_CHOOSE_CASE,MB_ICONWARNING|MB_OK);
		return;
	}

	srand(GetTickCount());

	switch(m_StartsWith)
	{
		// Letter
		case 0:
			m_NewPassword = szLetters[rand()%strlen(szLetters)];
			bUpper = (isupper(m_NewPassword.GetAt(0)));
			bLower = !bUpper;
			break;

		// Number
		case 1:
			m_NewPassword = szNumbers[rand()%strlen(szNumbers)];
			break;

		// Symbol
		case 2:
			m_NewPassword = szSymbols[rand()%strlen(szSymbols)];
			break;
	}

	// Figure out how many characters of each type we need and 
	// take into account both uppercase and lowercase letters
	nIndex = m_Letters+m_Numbers+m_Symbols;
	if(m_Letters && m_Upper && m_Lower) nIndex++;

	for(int i = 1; i < m_Length; i++)
	{
		char aszMix[4];
		int nCount = 0;

		if(m_Letters)
			aszMix[nCount++] = szLetters[rand()%strlen(szLetters)];
		
		if(m_Numbers)
			aszMix[nCount++] = szNumbers[rand()%strlen(szNumbers)];

		if(m_Symbols)
			aszMix[nCount++] = szSymbols[rand()%strlen(szSymbols)];

		if(m_Letters)
			aszMix[nCount++] = szLetters[rand()%strlen(szLetters)];

		// Get all of the required characters
		if(nIndex > 0)
		{
			nIndex--;
			ch = aszMix[i-1];

			// Make sure we end up with both uppercase and lowercase
			if(isalpha(ch))
			{
				if(!bUpper)
				{
					ch = toupper(ch);
					bUpper = TRUE;
				}
				else if(!bLower)
				{
					ch = tolower(ch);
					bLower = TRUE;
				}
			}

			m_NewPassword += ch;
		}
		else m_NewPassword += aszMix[rand()%nCount];
	}

	if(m_Upper && !m_Lower)
		m_NewPassword.MakeUpper();
	else if(m_Lower && !m_Upper)
		m_NewPassword.MakeLower();

	// Shuffle the characters around a little bit, to make sure
	// we don't have a fixed sequence of required characters
	for(i = 1; i < m_Length; i++)
	{
		// Get the current character
		ch = m_NewPassword.GetAt(i);

		// Get a random "swap" character, verify that index is > 0 because
		// we need to skip the start character
		nIndex = (rand() % (m_Length-1))+1;    
		ch2 = m_NewPassword.GetAt(nIndex);

		// Finally, swap the two characters
		m_NewPassword.SetAt(i,ch2);
		m_NewPassword.SetAt(nIndex,ch);
	}

	UpdateData(FALSE);

	m_OK.EnableWindow(TRUE);
}

void CPPGenPassDlg::OnOK() 
{
	CDialog::OnOK();
}

// Handles Help button
void CPPGenPassDlg::OnHelpBtn() 
{
	AfxGetApp()->WinHelp(101,HELP_CONTEXT);
}

// Saves settings for password screen even if the user cancels the
// screen
void CPPGenPassDlg::OnDestroy() 
{
	// Get the length and validate it so we won't end up with
	// an invalid amount
	m_Length = GetDlgItemInt(IDC_LENGTH);
	if(m_Length < 5) 
	{
		m_Length = 5;
		SetDlgItemInt(IDC_LENGTH,m_Length);
	}

	// Get the screen fields
	UpdateData(TRUE);

	// Save the profile settings
	AfxGetApp()->WriteProfileInt("Passgen","Length",m_Length);
	AfxGetApp()->WriteProfileInt("Passgen","StartsWith",m_StartsWith);
	AfxGetApp()->WriteProfileInt("Passgen","Upper",m_Upper);
	AfxGetApp()->WriteProfileInt("Passgen","Lower",m_Lower);;
	AfxGetApp()->WriteProfileInt("Passgen","Letters",m_Letters);;
	AfxGetApp()->WriteProfileInt("Passgen","Numbers",m_Numbers);;
	AfxGetApp()->WriteProfileInt("Passgen","Symbols",m_Symbols);;
	
	CDialog::OnDestroy();
}
