/*
	Password Prompter - PC Magazine password utility

	Copyright (c) 1999 Ziff-Davis Publishing Company.  All rights reserved.
	First published in PC Magazine, US Edition  June 8, 1999.

	Written by Steven E. Sipe


	This file implements the password edit control class.
*/

#include "stdafx.h"
#include "passprom.h"
#include "StarEdit.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CStarEdit

CStarEdit::CStarEdit()
{
}

CStarEdit::~CStarEdit()
{
}

BEGIN_MESSAGE_MAP(CStarEdit, CEdit)
	//{{AFX_MSG_MAP(CStarEdit)
	ON_WM_RBUTTONDOWN()
	ON_COMMAND(ID_CUT, OnCut)
	ON_COMMAND(ID_DELETE, OnDelete)
	ON_COMMAND(ID_PASTE, OnPaste)
	ON_COMMAND(ID_SELECT_ALL, OnSelectAll)
	ON_COMMAND(ID_UNDO, OnUndo)
	ON_WM_INITMENUPOPUP()
	ON_COMMAND(ID_COPY, OnCopy)
	//}}AFX_MSG_MAP
	ON_MESSAGE(WM_COPY,OnCopyKbd)
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CStarEdit message handlers

// Handles the copy to clipboard
void CStarEdit::DoCopy()
{
	// Open the clipboard
	if(OpenClipboard())
	{
		CString strText;
		int nStart, nEnd;

		// Get the edit control's text
		GetWindowText(strText);
		GetSel(nStart,nEnd);

		// Only save the selected text
		strText = strText.Mid(nStart,nEnd-nStart);

		HANDLE hText = GlobalAlloc(GMEM_MOVEABLE|GMEM_DDESHARE,strText.GetLength()+10);

		// Copy the data
		if(!strText.IsEmpty() && hText)
		{
			LPTSTR lpszText = (LPTSTR) GlobalLock(hText);

			strcpy(lpszText,strText);
			GlobalUnlock(hText);
			EmptyClipboard();
			SetClipboardData(CF_TEXT,hText);
		}

		// Close the clipboard
		CloseClipboard();
	}
}

// Handles the keyboard copy operation
LRESULT CStarEdit::OnCopyKbd(WPARAM wParam, LPARAM lParam)
{
	DoCopy();

	return(0);
}

// Handles the Copy menu item
void CStarEdit::OnCopy() 
{
	DoCopy();
}

// Handles the Cut menu item
void CStarEdit::OnCut() 
{
	DoCopy();

	Clear();
}

// Handles the Delete menu item
void CStarEdit::OnDelete() 
{
	Clear();
}

// Handles the Paste menu item
void CStarEdit::OnPaste() 
{
	Paste();
}

// Handles the Select All menu item
void CStarEdit::OnSelectAll() 
{
	SetSel(0,-1);
}

// Handles the Undo menu item
void CStarEdit::OnUndo() 
{
	if(CanUndo())
		Undo();
}

#define ENABLED() (MF_BYCOMMAND|MF_ENABLED)
#define DISABLED() (MF_BYCOMMAND|MF_DISABLED|MF_GRAYED)

// Enables and disables menu items when the menu is initially displayed
void CStarEdit::OnInitMenuPopup(CMenu* pPopupMenu, UINT nIndex, BOOL bSysMenu) 
{
	pPopupMenu->EnableMenuItem(ID_UNDO,CanUndo()?ENABLED():DISABLED());

	int nStart, nEnd;
	GetSel(nStart,nEnd);

	int nEnable = ((nEnd > nStart)?ENABLED():DISABLED());

	pPopupMenu->EnableMenuItem(ID_CUT,nEnable);
	pPopupMenu->EnableMenuItem(ID_COPY,nEnable);
	pPopupMenu->EnableMenuItem(ID_PASTE,nEnable);
	pPopupMenu->EnableMenuItem(ID_DELETE,nEnable);

	pPopupMenu->EnableMenuItem(ID_SELECT_ALL,(LineLength() > 0?ENABLED():DISABLED()));
}

// Displays the context menu
void CStarEdit::OnRButtonDown(UINT nFlags, CPoint point) 
{
	CMenu menu;
	menu.LoadMenu(IDR_CONTEXT_MENU);
	CMenu *pPopup = menu.GetSubMenu(0);

	SetFocus();
	ClientToScreen(&point);
	pPopup->TrackPopupMenu(TPM_LEFTALIGN|TPM_LEFTBUTTON|TPM_RIGHTBUTTON,
							point.x,point.y,this,NULL);
}

